//
//                                                                         
// Module: Internals Example Source File                                   
//                                                                         
// Description: Declarations for the Internals Example Plugin              
//                                                                         
//                                                                         
// This source code module, and all information, data, and algorithms      
// associated with it, are part of CUBE technology (tm).                   
//                 PROPRIETARY AND CONFIDENTIAL                            
// Copyright (c) 1996-2007 Image Space Incorporated.  All rights reserved. 
//                                                                         
//                                                                         
// Change history:                                                         
//   tag.2005.11.30: created                                               
//                                                                         
//

#include "Example.hpp"          // corresponding header file
#include <math.h>               // for atan2, sqrt
#include <stdio.h>              // for sample output




// plugin information
unsigned g_uPluginID          = 0;
char     g_szPluginName[]     = "SimToolsPlugin - 2013.31.08";
unsigned g_uPluginVersion     = 001;
unsigned g_uPluginObjectCount = 1;
InternalsPluginInfo g_PluginInfo;

// interface to plugin information
extern "C" __declspec(dllexport)
const char* __cdecl GetPluginName() { return g_szPluginName; }

extern "C" __declspec(dllexport)
unsigned __cdecl GetPluginVersion() { return g_uPluginVersion; }

extern "C" __declspec(dllexport)
unsigned __cdecl GetPluginObjectCount() { return g_uPluginObjectCount; }

// get the plugin-info object used to create the plugin.
extern "C" __declspec(dllexport)
PluginObjectInfo* __cdecl GetPluginObjectInfo( const unsigned uIndex )
{
  switch(uIndex)
  {
    case 0:
      return  &g_PluginInfo;
    default:
      return 0;
  }
}


// InternalsPluginInfo class

   HANDLE hMapFile;
   LPVOID pBuf;
   const DWORD BUF_SIZE = 780 + 492 + 100 * 428; //Size of telemetry and scoring data + 100 divers' VehicleScoringInfoV2
   
InternalsPluginInfo::InternalsPluginInfo()
{
  // put together a name for this plugin
  sprintf( m_szFullName, "%s - %s", g_szPluginName, InternalsPluginInfo::GetName() );
}

const char*    InternalsPluginInfo::GetName()     const { return ExampleInternalsPlugin::GetName(); }
const char*    InternalsPluginInfo::GetFullName() const { return m_szFullName; }
const char*    InternalsPluginInfo::GetDesc()     const { return "Example Internals Plugin"; }
const unsigned InternalsPluginInfo::GetType()     const { return ExampleInternalsPlugin::GetType(); }
const char*    InternalsPluginInfo::GetSubType()  const { return ExampleInternalsPlugin::GetSubType(); }
const unsigned InternalsPluginInfo::GetVersion()  const { return ExampleInternalsPlugin::GetVersion(); }
void*          InternalsPluginInfo::Create()      const { return new ExampleInternalsPlugin(); }


// InternalsPlugin class

const char ExampleInternalsPlugin::m_szName[] = "InternalsPlugin";
const char ExampleInternalsPlugin::m_szSubType[] = "Internals";
const unsigned ExampleInternalsPlugin::m_uID = 1;
const unsigned ExampleInternalsPlugin::m_uVersion = 3;  // set to 3 for InternalsPluginV3 functionality and added graphical and vehicle info


PluginObjectInfo *ExampleInternalsPlugin::GetInfo()
{
  return &g_PluginInfo;
}


void ExampleInternalsPlugin::WriteToAllExampleOutputFiles( const char * const openStr, const char * const msg )
{}


void ExampleInternalsPlugin::Startup()
{
   const char* szName = "SimToolsrFactor";
   hMapFile = CreateFileMapping(INVALID_HANDLE_VALUE,NULL,PAGE_READWRITE, 0,BUF_SIZE,szName);
   pBuf = MapViewOfFile(hMapFile,FILE_MAP_ALL_ACCESS, 0,0,BUF_SIZE);
}

void ExampleInternalsPlugin::Shutdown()
{
   UnmapViewOfFile(pBuf);
   CloseHandle(hMapFile);
}

void ExampleInternalsPlugin::StartSession()
{}

void ExampleInternalsPlugin::EndSession()
{}

void ExampleInternalsPlugin::EnterRealtime()
{}

void ExampleInternalsPlugin::ExitRealtime()
{}

void ExampleInternalsPlugin::UpdateTelemetry( const TelemInfoV2 &info )
{
	CopyMemory(pBuf, &info, 780); //TelemInfoV2
}


void ExampleInternalsPlugin::UpdateGraphics( const GraphicsInfoV2 &info )
{}


bool ExampleInternalsPlugin::CheckHWControl( const char * const controlName, float &fRetVal )
{  return( false );}


bool ExampleInternalsPlugin::ForceFeedback( float &forceValue )
{  return( false );}


void ExampleInternalsPlugin::UpdateScoring( const ScoringInfoV2 &info )
{
	CopyMemory((byte*)pBuf +780, &info, 492); //ScoringInfoV2 
	CopyMemory( (byte*)pBuf +1272, info.mVehicle, info.mNumVehicles*428); //VehicleScoringInfoV2
}


bool ExampleInternalsPlugin::RequestCommentary( CommentaryRequestInfo &info )
{  return( false );}

